pub enum MatchType {
Simple(EmitSimple),
Open(Arc<ScopeInfo>),
Close,
}Expand description
This defines how scopes and whole blocks of text move into the output. Basically: is your matcher
using scope or not? If using scope, you need to define an open and close MatchInfo. If not,
just use the [Simple] option
Variants
Simple(EmitSimple)
A match type that requires no scoping rules: just a simple regex replacement (or whatever replacement you want)
Open(Arc<ScopeInfo>)
The match should expect an open tag, which increases scope and performs open scope rules
Close
The match should expect a closing tag, which decreases scope and performs close scope rules