pub struct MatchInfo {
pub id: &'static str,
pub regex: Regex,
pub match_type: MatchType,
}Expand description
Definition for a block level matcher. Should always be readonly, it is just a definition. Not necessarily a scoped element, could define eating garbage, passing normal text through, etc. It’s all up to the ‘match_type’
Fields
id: &'static strA unique identifier for you to reference in things like [ScopeInfo.only]
regex: RegexThe regex for this parse item. Most likely an open or close tag, but you can do anything you want
match_type: MatchTypeThe type of match, indicates whether to open or close a scope (or perform no scoping)